@@ -4,6 +4,8 @@
 #!# Copyright (c) 2022, Achim Hoffmann
 #!# This software is licensed under GPLv2.  Please see o-saft.pl for details.
 
+package OSaft::Doc::Data;
+
 ## no critic qw(Documentation::RequirePodSections)
 #        Our POD below is fine, Perl::Critic (severity 2) is too pedantic here.
 
@@ -19,23 +21,25 @@
 #        to have simple and complex regex.  /x is used for human readability as
 #        needed.
 
-package OSaft::Doc::Data;
-
 use strict;
 use warnings;
 
 BEGIN { # mainly required for testing ...
+    # SEE Perl:@INC
     # SEE Perl:BEGIN perlcritic
-    my $_path = $0;     $_path =~ s#[/\\][^/\\]*$##;
-    unshift(@INC, ".",  $_path);
+    my $_me   = $0;     $_me   =~ s#.*[/\\]##x;
+    my $_path = $0;     $_path =~ s#[/\\][^/\\]*$##x;
+    unshift(@INC, $_path)   if (1 > (grep{/^$_path$/} @INC));
+    unshift(@INC, "..")     if (1 > (grep{/^\.\.$/}   @INC));
+    unshift(@INC, ".")      if (1 > (grep{/^\.$/}     @INC));
 }
 
-our $VERSION    = "22.02.13";  # official verion number of tis file
-my  $SID_data   = "@(#) Data.pm 1.45 22/02/18 20:57:31";
+my  $SID_data   = "@(#) Data.pm 1.56 22/11/04 00:30:15";
+our $VERSION    = "22.11.22";   # official verion number of this file
 
 # binmode(...); # inherited from parent, SEE Perl:binmode()
 
-use osaft qw(print_pod STR_WARN);
+use OSaft::Text qw(print_pod);
 
 #_____________________________________________________________________________
 #_____________________________________________________ public documentation __|
@@ -44,10 +48,12 @@ use osaft qw(print_pod STR_WARN);
 
 =encoding utf8
 
+
 =head1 NAME
 
 OSaft::Doc::Data - common Perl module to read data for user documentation
 
+
 =head1 SYNOPSIS
 
 =over 2
@@ -60,18 +66,14 @@ OSaft::Doc::Data - common Perl module to read data for user documentation
 
 =back
 
-
-=head1 METHODS
-
 =cut
 
 #_____________________________________________________________________________
 #_________________________________________________________ internal methods __|
 
 # SEE Perl:Undefined subroutine
-*_warn = sub { print(join(" ", "**WARNING:", @_), "\n"); return; } if not defined &_warn;
-*_dbx  = sub { print(join(" ", "#dbx#"     , @_), "\n"); return; } if not defined &_dbx;
-# TODO: return if (grep{/(?:--no.?warn)/} @ARGV);   # ugly hack
+*_warn    = sub { print(join(" ", "**WARNING:", @_), "\n"); return; } if not defined &_warn;
+*_dbx     = sub { print(join(" ", "#dbx#"     , @_), "\n"); return; } if not defined &_dbx;
 
 sub _replace_var    {
     #? replace $0 by name and $VERSION by version in array, return array
@@ -91,6 +93,7 @@ sub _get_standalone {
     $file =~ s#contrib/##;              # remove if in path
     $file =  "OSaft/Doc/$file";         # try this one ..
     $file =  "../$file" if (not -e $file);  # .. or this one
+    $file =  ""         if (not -e $file);
     return $file;
 } # _get_standalone
 
@@ -148,6 +151,8 @@ sub get_egg     {
 
 =pod
 
+=head1 METHODS
+
 =head2 get($file,$name,$version)
 
 Return all data from file and replace $0 by $name. Returns data as string.
@@ -155,6 +160,7 @@ Return all data from file and replace $0 by $name. Returns data as string.
 =cut
 
 sub get         {
+    #? return data from file as string, replace $0 by $name
     my $file    = shift;
     my $name    = shift || "o-saft.pl";
     my $version = shift || $VERSION;
@@ -183,11 +189,13 @@ Return all data converted to internal markup format. Returns array of lines.
 =cut
 
 sub get_markup    {
+    #? return data with internal markup, returns array of lines
     my $file    = shift;
     my $parent  = shift || "o-saft.pl";
     my $version = shift || $VERSION;
     my @txt;
     my $fh      = _get_filehandle($file);
+    return "" if ("" eq $fh);           # defensive programming
     # Preformat plain text with markup for further simple substitutions. We
     # use a modified  &  instead of < >  POD markup as it is easy to parse.
     # &  was choosen because it rarely appears in texts and  is not  a meta
@@ -407,7 +415,20 @@ sub list        {
     return join("\n", sort @txt);
 } # list
 
-sub _main_usage {
+#_____________________________________________________________________________
+#____________________________________________________ internal test methods __|
+
+# none, piblic methods used directly
+
+#_____________________________________________________________________________
+#___________________________________________________ initialisation methods __|
+
+# none
+
+#_____________________________________________________________________________
+#_____________________________________________________________________ main __|
+
+sub _main_doc_usage {
     #? print usage
     my $name = (caller(0))[1];
     print "# various commands:\n";
@@ -421,9 +442,9 @@ sub _main_usage {
     }
     printf("\t$name ciphers=dumptab > c.csv; libreoffice c.csv\n");
     return;
-}; # _main_usage
+}; # _main_doc_usage
 
-sub _main       {
+sub _main_doc   {
     #? print own documentation or that from specified file
     ## no critic qw(InputOutput::RequireEncodingWithUTF8Layer)
     #  see t/.perlcriticrc for detailed description of "no critic"
@@ -436,7 +457,7 @@ sub _main       {
     while (my $cmd = shift @argv) {
         my $arg    = shift @argv; # get 2nd argument, which is filename
         print_pod($0, __PACKAGE__, $SID_data)   if ($cmd =~ /^--?h(?:elp)?$/);
-        _main_usage()           if ($cmd =~ /^--usage$/);
+        _main_doc_usage()       if ($cmd eq '--usage');
         # ----------------------------- commands
         print list() . "\n"     if ($cmd =~ /^list$/);
         print get($arg)         if ($cmd =~ /^get$/);
@@ -448,9 +469,12 @@ sub _main       {
         print "$VERSION\n"      if ($cmd =~ /^[-+]?V(ERSION)?$/);
     }
     exit 0;
-} # _main
+} # _main_doc
 
-sub o_saft_help_done {};    # dummy to check successful include
+sub doc_data_done   {}; # dummy to check successful include
+
+#_____________________________________________________________________________
+#_____________________________________________________ public documentation __|
 
 =pod
 
@@ -589,13 +613,16 @@ Means that the prefixes  "Commands for"  and  "Options for"  are used to
 identify groups of commands and options. If a sub-title does not start
 with these prefixes, all following commands and options are ignored.
 
+
 =head1 SEE ALSO
 
 # ...
 
+
 =head1 VERSION
 
-1.45 2022/02/18
+1.56 2022/11/04
+
 
 =head1 AUTHOR
 
@@ -608,15 +635,17 @@ with these prefixes, all following commands and options are ignored.
 #_____________________________________________________________________________
 #_____________________________________________________________________ self __|
 
-_main(@ARGV) if (not defined caller);
+_main_doc(@ARGV) if (not defined caller);
 
 1;
 
+# __END__
+    # __end__ not possible, because we may get
+    # readline() on unopened filehandle DATA at OSaft/Doc/Data.pm line 139.
+
 # SEE Note:Documentation
 # All public (user) documentation is in plain ASCII format (see help.txt).
 
-__END__
-
 =pod
 
 =head1 Annotations, Internal Notes
